// Mutes or unmutes a portion of an audio clip.
// To use:
// 1. Select the desired audio track
// 2. Define a time selection for the part of the clip to mute
// 3. Run the script
// The script creates a mute or volume envelope for the selected track if one does not exist and adds points
// to mute the audio. If points already exist in the selection, they are deleted to unmute the audio.
//
// Author: Randall Campbell, info@peachrock.com, www.peachrock.com
//  Copyright 2004, Peach Rock Productions, LLC.
// You are free to use or modify this code as long as the copyright information is not removed.
// This software is provided AS IS, no warranty is expressed or implied

import System;
import System.Windows.Forms;
import System.Collections;
import Sony.Vegas;

// You can change the fade amount if desired. The default is 1 milliseconds. This value must be greater than 0.
var fadeOut : Timecode = Timecode.FromMilliseconds(1);
var fadeIn : Timecode = Timecode.FromMilliseconds(1);

// You can modify the curve type here. Valid values are: Smooth, Sharp, Slow, Linear, Fast, None (ie. Hold)
var fadeOutCurve : CurveType = CurveType.Smooth;
var fadeInCurve : CurveType = CurveType.Smooth;

// You can change the envelope to use the volume envelope by changing EnvelopeType.Mute to EnvelopeType.Volume
// Just comment out the mode that you don't want
//var envelopeType : EnvelopeType = EnvelopeType.Mute;
var envelopeType : EnvelopeType = EnvelopeType.Volume;

for (var track in Vegas.Project.Tracks)
{
    if (!track.IsAudio() || !track.Selected)
	continue;

    if (!track.Envelopes.HasEnvelope(envelopeType))
        track.Envelopes.Add(new Envelope(envelopeType));

    var envelope : Envelope = track.Envelopes.FindByType(envelopeType);

    var unmuted = false;
    for (var i = envelope.Points.Count - 1; i > 0; i--)
    {
        var point : EnvelopePoint = envelope.Points.Item[i];
        if (point.X >= Vegas.SelectionStart && point.X <= (Vegas.SelectionStart + Vegas.SelectionLength))
	    {
	    envelope.Points.Remove(point);
	    unmuted = true;
	    }
    }
    if (!unmuted)
    {
        envelope.Points.Add(new EnvelopePoint(Vegas.SelectionStart, 1, fadeOutCurve));
        envelope.Points.Add(new EnvelopePoint(Vegas.SelectionStart + fadeOut, 0));
        envelope.Points.Add(new EnvelopePoint(Vegas.SelectionStart + Vegas.SelectionLength - fadeIn, 0, fadeInCurve));
        envelope.Points.Add(new EnvelopePoint(Vegas.SelectionStart + Vegas.SelectionLength, 1));
    }
}


